pub use spacemesh_api_client as client;
pub use spacemesh_api_client::Client;
pub use spacemesh_codec as codec;
pub use spacemesh_codec::tx_types as transaction;


pub mod wallet;

pub mod bech32 {
    pub use bech32::*;

    pub fn encode(hrp: Hrp, input: &[u8]) -> Result<String, EncodeError> {
        bech32::encode::<Bech32>(hrp, input)
    }
}

#[cfg(test)]
mod tests {
    use super::*;
    use base64::prelude::*;
    use spacemesh_api_client::*;

    #[tokio::test]
    async fn decodes_live_transaction() {
        let client = Client::new("https://mainnet-api.spacemesh.network");
        let txid = "638442a2033f20b5a7280b9a4f2bfc73022f6e7ec64b1497b85335444381d99d";
        let txid = smex::decode(txid).unwrap();
        let txid = BASE64_STANDARD.encode(txid);
        let result = client
            .transaction_service_list(&types::Spacemeshv2alpha1TransactionRequest {
                txid: vec![txid],
                limit: Some(100.to_string()),
                ..Default::default()
            })
            .await
            .unwrap()
            .into_inner();

        let mut result = match result {
            types::GooglerpcStatusOrSpacemeshv2alpha1TransactionList::GooglerpcStatus(googlerpc_status) => panic!("{:?}", googlerpc_status.message),
            types::GooglerpcStatusOrSpacemeshv2alpha1TransactionList::Spacemeshv2alpha1TransactionList(transaction_list) => {
                transaction_list
            },
        };

        let tx = result.transactions.pop().unwrap().tx.unwrap();
        let (_hrp, address) = bech32::decode(&tx.template.unwrap()).unwrap();
        let tx_raw = BASE64_STANDARD.decode(tx.raw.unwrap()).unwrap();
        let decoded = transaction::decode_by_address_and_method(
            address.try_into().unwrap(),
            tx.method.unwrap() as u8,
            &tx_raw,
        )
        .unwrap()
        .unwrap();
        drop(decoded);
    }
}
